@@ -73,7 +73,7 @@ def do_versioning(latest):
     ver_changes = dict()
     
     if c_updates:
-        # denote pdfium update (independent change)
+        # denote pdfium update
         ver_changes["V_LIBPDFIUM"] = str(latest)
     
     if inc_major:
@@ -84,7 +84,7 @@ def do_versioning(latest):
         os.remove(MajorUpdateFile)
     elif v_beta is None:
         # if we're not doing a major update and the previous version was not a beta, update minor and/or patch
-        # however, we still want to run this if adding a new beta tag
+        # (note that we still want to run this if adding a new beta tag, though)
         if c_updates:
             # pdfium update -> increment minor version and reset patch version
             ver_changes["V_MINOR"] = VerNamespace["V_MINOR"] + 1
@@ -94,14 +94,14 @@ def do_versioning(latest):
             ver_changes["V_PATCH"] = VerNamespace["V_PATCH"] + 1
     
     if inc_beta:
-        # if the new version shall be a beta, set or increment the tag (independent change)
+        # if the new version shall be a beta, set or increment the tag
         if v_beta is None:
             v_beta = 0
         v_beta += 1
         ver_changes["V_BETA"] = v_beta
         os.remove(BetaUpdateFile)
     elif v_beta is not None:
-        # if the previous version was a beta but the new one shall not be, remove the tag (independent change)
+        # if the previous version was a beta but the new one shall not be, remove the tag
         ver_changes["V_BETA"] = None
     
     did_change = set_versions(ver_changes)
@@ -134,12 +134,11 @@ def log_changes(summary, prev_ns, curr_ns):
 
 
 def register_changes(curr_ns):
+    run_local(["git", "checkout", "-B", "autorelease_tmp", "main"])
     run_local(["git", "add", AutoreleaseDir, VersionFile, Changelog, ChangelogStaging])
     run_local(["git", "commit", "-m", "[autorelease] update changelog and version file"])
+    # NOTE the actually pushed tag will be a different one, but it's nevertheless convenient to have this here because of the changelog
     run_local(["git", "tag", "-a", curr_ns["V_PYPDFIUM2"], "-m", "Autorelease"])
-    run_local(["git", "checkout", "stable"])
-    run_local(["git", "reset", "--hard", "main"])
-    run_local(["git", "checkout", "main"])
 
 
 def _get_log(name, url, cwd, ver_a, ver_b, prefix_ver, prefix_commit, prefix_tag):
@@ -154,7 +153,7 @@ def _get_log(name, url, cwd, ver_a, ver_b, prefix_ver, prefix_commit, prefix_tag
     log += run_cmd(["git", "log",
         "%s..%s" % (prefix_tag+ver_a, prefix_tag+ver_b),
         "--pretty=format:* [`%h`]({}%H) %s".format(url+prefix_commit)],
-        capture=True, cwd=cwd,
+        capture=True, check=False, cwd=cwd,
     )
     log += "\n\n</details>\n"
     return log
@@ -197,9 +196,9 @@ def main():
         description = "Automatic update script for pypdfium2, to be run in the CI release workflow."
     )
     parser.add_argument(
-        "--checkin",
+        "--register",
         action = "store_true",
-        help = "Allow running modifying git commands (commit, tag, reset)."
+        help = "Save changes in autorelease_tmp branch."
     )
     args = parser.parse_args()
     
@@ -212,7 +211,7 @@ def main():
         flush = (curr_ns["V_BETA"] is None),
     )
     log_changes(summary, prev_ns, curr_ns)
-    if args.checkin:
+    if args.register:
         register_changes(curr_ns)
     make_releasenotes(summary, prev_ns, curr_ns, c_updates)
 
