@@ -2,6 +2,8 @@
 # SPDX-FileCopyrightText: 2022 geisserml <geisserml@gmail.com>
 # SPDX-License-Identifier: Apache-2.0 OR BSD-3-Clause
 
+# NOTE Works on Linux/macOS/Windows (that is, at least on GitHub Actions)
+
 import os
 import sys
 import shutil
@@ -15,7 +17,6 @@ from pl_setup.packaging_base import (
     DataTree,
     PDFium_URL,
     DepotTools_URL,
-    MainLibnames,
     LibnameForSystem,
     VerStatusFileName,
     PlatformNames,
@@ -31,8 +32,8 @@ PDFiumBuildDir = join(PDFiumDir, "out", "Default")
 OutputDir      = join(DataTree, PlatformNames.sourcebuild)
 
 PdfiumMainPatches = [
-    (join(PatchDir, "public_headers.patch"), PDFiumDir),
     (join(PatchDir, "shared_library.patch"), PDFiumDir),
+    (join(PatchDir, "public_headers.patch"), PDFiumDir),
 ]
 PdfiumWinPatches = [
     (join(PatchDir, "win", "pdfium.patch"), PDFiumDir),
@@ -50,9 +51,7 @@ DefaultConfig = {
 }
 
 if sys.platform.startswith("darwin"):
-    DefaultConfig["mac_deployment_target"] = "10.11.0"
-elif sys.platform.startswith("win32"):
-    DefaultConfig["pdf_use_win32_gdi"] = True
+    DefaultConfig["mac_deployment_target"] = "10.13.0"
 
 
 def dl_depottools(do_update):
@@ -94,10 +93,8 @@ def dl_pdfium(GClient, do_update, revision):
         print("PDFium: Download ...")
         run_cmd([GClient, "config", "--custom-var", "checkout_configuration=minimal", "--unmanaged", PDFium_URL], cwd=SB_Dir)
     
-    # TODO consider --with_branch_heads so we can improve get_pdfium_version()
-    # while the heads take up additional disk space, this shouldn't be a problem for a shallow clone
     if is_sync:
-        run_cmd([GClient, "sync", "--revision", "origin/%s" % revision, "--no-history", "--shallow"], cwd=SB_Dir)
+        run_cmd([GClient, "sync", "--revision", "origin/%s" % revision, "--no-history", "--with_branch_heads"], cwd=SB_Dir)
     
     return is_sync
 
@@ -130,11 +127,6 @@ def update_version(v_libpdfium):
         fh.write( str(v_libpdfium) )
 
 
-def _apply_patchset(patchset):
-    for patch, cwd in patchset:
-        run_cmd(["git", "apply", "-v", patch], cwd=cwd)
-
-
 def _create_resources_rc(v_libpdfium):
     
     input_path = join(PatchDir, "win", "resources.rc")
@@ -143,14 +135,22 @@ def _create_resources_rc(v_libpdfium):
     with open(input_path, "r") as fh:
         content = fh.read()
     
-    vars = ("$VERSION_CSV", "$VERSION")
-    for var in vars:
-        content = content.replace(var, v_libpdfium)
+    # NOTE RC does not seem to tolerate commit hash, so set a dummy version instead
+    if not v_libpdfium.isnumeric():
+        v_libpdfium = "1.0"
+    
+    content = content.replace("$VERSION_CSV", v_libpdfium.replace(".", ","))
+    content = content.replace("$VERSION", v_libpdfium)
     
     with open(output_path, "w") as fh:
         fh.write(content)
 
 
+def _apply_patchset(patchset):
+    for patch, cwd in patchset:
+        run_cmd(["git", "apply", "--ignore-space-change", "--ignore-whitespace", "-v", patch], cwd=cwd)
+
+
 def patch_pdfium(v_libpdfium):
     _apply_patchset(PdfiumMainPatches)
     if sys.platform.startswith("win32"):
@@ -179,15 +179,20 @@ def find_lib(src_libname=None, directory=PDFiumBuildDir):
         else:
             print("Warning: Binary not found under given name.", file=sys.stderr)
     
-    try_names = []
-    for name in MainLibnames + ["pdfium.so"]:
-        try_names += [name, "lib"+name]
+    if sys.platform.startswith("linux"):
+        libname = "libpdfium.so"
+    elif sys.platform.startswith("darwin"):
+        libname = "libpdfium.dylib"
+    elif sys.platform.startswith("win32"):
+        libname = "pdfium.dll"
+    else:
+        # TODO implement fallback artifact detection
+        raise RuntimeError("Not sure how pdfium artifact is called on platform '%s'" % (sys.platform, ))
     
-    try_paths = [join(directory, n) for n in try_names]
-    found_paths = [fp for fp in try_paths if os.path.isfile(fp)]
+    libpath = join(directory, libname)
+    assert os.path.exists(libpath)
     
-    assert len(found_paths) == 1
-    return found_paths[0]
+    return libpath
 
 
 def pack(src_libpath, v_libpdfium, destname=None):
@@ -249,6 +254,9 @@ def main(
     
     if sys.platform.startswith("win32"):
         os.environ["DEPOT_TOOLS_WIN_TOOLCHAIN"] = "0"
+        WindowsSDK_DIR = R"C:\Program Files (x86)\Windows Kits\10\bin\10.0.19041.0\x64"
+        assert os.path.isdir(WindowsSDK_DIR)
+        os.environ["PATH"] += os.pathsep + WindowsSDK_DIR
     
     dl_depottools(b_update)
     
