@@ -4,22 +4,25 @@
 import io
 import os
 import os.path
+import weakref
 import ctypes
 import logging
 import functools
 from concurrent.futures import ProcessPoolExecutor
 
 import pypdfium2._pypdfium as pdfium
-from pypdfium2._helpers._opener import (
-    open_pdf,
-    is_input_buffer,
-)
 from pypdfium2._helpers.misc import (
     OutlineItem,
     FileAccess,
     PdfiumError,
+    ErrorToStr,
     ViewmodeToStr,
     get_functype,
+    get_fileaccess,
+    is_input_buffer,
+)
+from pypdfium2._helpers.pageobject import (
+    PdfPageObject,
 )
 from pypdfium2._helpers.converters import BitmapConvAliases
 from pypdfium2._helpers.page import PdfPage
@@ -39,7 +42,7 @@ class PdfDocument (BitmapConvAliases):
     Parameters:
         input_data (str | bytes | typing.BinaryIO | FPDF_DOCUMENT):
             The input PDF given as file path, bytes, byte buffer, or raw PDFium document handle.
-            A byte buffer is defined as an object that implements the methods ``seek()``, ``tell()``, ``read()`` and ``readinto()``.
+            :func:`.is_input_buffer` defines which objects are recognised as byte buffers.
         password (str | bytes):
             A password to unlock the PDF, if encrypted.
             If the document is not encrypted but a password was given, PDFium will ignore it.
@@ -54,7 +57,6 @@ class PdfDocument (BitmapConvAliases):
     
     Hint:
         * :func:`len` may be called to get a document's number of pages.
-        * :class:`PdfDocument` implements the context manager API, hence documents can be used in a ``with`` block, where :meth:`.close` will be called automatically on exit.
         * Looping over a document will yield its pages from beginning to end.
         * Pages may be loaded using list index access.
         * The ``del`` keyword and list index access may be used to delete pages.
@@ -73,15 +75,18 @@ class PdfDocument (BitmapConvAliases):
         
         self._orig_input = input_data
         self._actual_input = input_data
+        self._data_holder = []
+        self._data_closer = []
         self._rendering_input = None
-        self._ld_data = None
-        self._form_env = None
-        self._form_config = None
         
         self._password = password
         self._file_access = file_access
         self._autoclose = autoclose
         
+        self._form_env = None
+        self._form_config = None
+        self._form_finalizer = None
+        
         if isinstance(self._orig_input, str):
             
             self._orig_input = os.path.abspath( os.path.expanduser(self._orig_input) )
@@ -92,7 +97,7 @@ class PdfDocument (BitmapConvAliases):
                 pass
             elif self._file_access is FileAccess.BUFFER:
                 self._actual_input = open(self._orig_input, "rb")
-                self._autoclose = True
+                self._data_closer.append(self._actual_input)
             elif self._file_access is FileAccess.BYTES:
                 buf = open(self._orig_input, "rb")
                 self._actual_input = buf.read()
@@ -103,14 +108,25 @@ class PdfDocument (BitmapConvAliases):
         if isinstance(self._actual_input, pdfium.FPDF_DOCUMENT):
             self.raw = self._actual_input
         else:
-            self.raw, self._ld_data = open_pdf(self._actual_input, self._password)
+            self.raw, ld_data = _open_pdf(self._actual_input, self._password)
+            self._data_holder += ld_data
+        
+        if self._autoclose and is_input_buffer(self._actual_input):
+            self._data_closer.append(self._actual_input)
+        
+        self._finalizer = weakref.finalize(
+            self, self._static_close,
+            self.raw, self._data_holder, self._data_closer,
+        )
     
     
     def __enter__(self):
         return self
     
     def __exit__(self, *_):
-        self.close()
+        # We do not invoke close at this place anymore because that would increase the risk of callers closing parent objects before child objects.
+        # (Consider a `with`-block where pages are not closed explicitly: garbage collection of pages commonly happens later than context manager exit, so page would be closed after document, which is illegal.)
+        pass
     
     def __len__(self):
         return pdfium.FPDF_GetPageCount(self.raw)
@@ -135,21 +151,45 @@ class PdfDocument (BitmapConvAliases):
         new_pdf = pdfium.FPDF_CreateNewDocument()
         return cls(new_pdf)
     
+    
+    @staticmethod
+    def _static_close(raw, data_holder, data_closer):
+        
+        # logger.debug("Closing document")
+        pdfium.FPDF_CloseDocument(raw)
+        
+        for data in data_holder:
+            id(data)
+        for data in data_closer:
+            data.close()
+    
+    
+    @staticmethod
+    def _static_exit_formenv(form_env, form_config):
+        # logger.debug("Closing form env")
+        pdfium.FPDFDOC_ExitFormFillEnvironment(form_env)
+        id(form_config)
+    
+    
     def close(self):
         """
-        Close the document to release allocated memory.
-        This function shall be called when finished working with the object.
+        Free memory by applying the finalizer for the underlying PDFium document.
+        Please refer to the generic note on ``close()`` methods for details.
+        
+        This method calls :meth:`.exit_formenv`.
         """
-        
+        if self.raw is None:
+            logger.warning("Duplicate close call suppressed on document %s" % self)
+            return
         self.exit_formenv()
-        
-        pdfium.FPDF_CloseDocument(self.raw)
+        self._finalizer()
         self.raw = None
-        
-        if self._ld_data is not None:
-            self._ld_data.close()
-        if self._autoclose and is_input_buffer(self._actual_input):
-            self._actual_input.close()
+        self._data_holder = []
+        self._data_closer = []
+    
+    
+    def _tree_closed(self):
+        return (self.raw is None)
     
     
     def init_formenv(self):
@@ -165,21 +205,23 @@ class PdfDocument (BitmapConvAliases):
         self._form_config = pdfium.FPDF_FORMFILLINFO()
         self._form_config.version = 2
         self._form_env = pdfium.FPDFDOC_InitFormFillEnvironment(self.raw, self._form_config)
+        self._form_finalizer = weakref.finalize(
+            self, self._static_exit_formenv,
+            self._form_env, self._form_config,
+        )
         return self._form_env
     
     
     def exit_formenv(self):
         """
-        Release allocated memory by exiting the form environment.
-        If the form environment is not initialised, nothing will be done.
+        Free memory by applying the finalizer for the underlying PDFium form environment, if it was initialised.
+        If :meth:`.init_formenv` was not called, nothing will be done.
         
-        Note:
-            This method is called by :meth:`.close`.
+        This behaves like the ``close()`` methods. Please refer to the generic note for details.
         """
         if self._form_env is None:
             return
-        pdfium.FPDFDOC_ExitFormFillEnvironment(self._form_env)
-        id(self._form_config)
+        self._form_finalizer()
         self._form_env = None
         self._form_config = None
     
@@ -236,7 +278,6 @@ class PdfDocument (BitmapConvAliases):
     def get_page_size(self, index):
         """
         Get the dimensions of the page at *index*. Reverse indexing is allowed.
-        This method is equivalent to :meth:`.PdfPage.get_size()` but works without needing to load a separate page object.
         
         Returns:
             (float, float): Page width and height in PDF canvas units.
@@ -246,7 +287,30 @@ class PdfDocument (BitmapConvAliases):
         success = pdfium.FPDF_GetPageSizeByIndexF(self.raw, index, size)
         if not success:
             raise PdfiumError("Getting page size by index failed.")
-        return (float(size.width), float(size.height))
+        return (size.width, size.height)
+    
+    
+    def page_as_xobject(self, index, dest_pdf):
+        """
+        Capture a page as XObject and attach it to a document's resources.
+        
+        Parameters:
+            index (int): Zero-based index of the page. Reverse indexing is allowed.
+            dest_pdf (PdfDocument): Target document to which the XObject shall be added.
+        Returns:
+            PdfXObject: The page as XObject.
+        """
+        
+        index = self._handle_index(index)
+        
+        raw_xobject = pdfium.FPDF_NewXObjectFromPage(dest_pdf.raw, self.raw, index)
+        if raw_xobject is None:
+            raise PdfiumError("Failed to capture page %s as FPDF_XOBJECT" % index)
+        
+        return PdfXObject(
+            raw = raw_xobject,
+            pdf = dest_pdf,
+        )
     
     
     def new_page(self, width, height, index=None):
@@ -459,7 +523,6 @@ class PdfDocument (BitmapConvAliases):
         )
         page = pdf.get_page(index)
         result = page.render_to(converter, **kwargs)
-        for g in (page, pdf): g.close()
         return result, index
     
     
@@ -473,7 +536,7 @@ class PdfDocument (BitmapConvAliases):
         """
         Concurrently render multiple pages, using a process pool executor.
         
-        .. seealso:: :meth:`.PdfPage.render_to` / :meth:`.PdfPage.render_base`
+        If rendering only a single page, the call is simply forwarded to :meth:`.PdfPage.render_to` as a shortcut.
         
         Parameters:
             page_indices (typing.Sequence[int] | None):
@@ -482,11 +545,28 @@ class PdfDocument (BitmapConvAliases):
             n_processes (int):
                 Target number of parallel processes.
             kwargs (dict):
-                Keyword arguments to the renderer.
+                Keyword arguments to the renderer. See :meth:`.PdfPage.render_to` / :meth:`.PdfPage.render_base`.
+        
         Yields:
             :data:`typing.Any`: Implementation-specific result object.
         """
         
+        n_pages = len(self)
+        if not page_indices:
+            page_indices = [i for i in range(n_pages)]
+        else:
+            if not all(0 <= i < n_pages for i in page_indices):
+                raise ValueError("Out-of-bounds page index")
+            if len(page_indices) != len(set(page_indices)):
+                raise ValueError("Duplicate page index")
+        
+        # shortcut: if we're rendering just a single page, don't waste time setting up a process pool
+        if len(page_indices) == 1:
+            page = self.get_page(page_indices[0])
+            result = page.render_to(converter, **kwargs)
+            yield result
+            return
+        
         if self._rendering_input is None:
             if isinstance(self._orig_input, pdfium.FPDF_DOCUMENT):
                 logger.warning("Cannot perform concurrent processing without input sources - saving the document implicitly to get picklable data.")
@@ -500,15 +580,6 @@ class PdfDocument (BitmapConvAliases):
             else:
                 self._rendering_input = self._orig_input
         
-        n_pages = len(self)
-        if not page_indices:
-            page_indices = [i for i in range(n_pages)]
-        else:
-            if not all(0 <= i < n_pages for i in page_indices):
-                raise ValueError("Out-of-bounds page index")
-            if len(page_indices) != len(set(page_indices)):
-                raise ValueError("Duplicate page index")
-        
         invoke_renderer = functools.partial(
             PdfDocument._process_page,
             converter = converter,
@@ -528,6 +599,31 @@ class PdfDocument (BitmapConvAliases):
         assert len(page_indices) == i
 
 
+def _open_pdf(input_data, password=None):
+    
+    if isinstance(password, str):
+        password = password.encode("utf-8")
+    
+    ld_data = ()
+    if isinstance(input_data, str):
+        pdf = pdfium.FPDF_LoadDocument(input_data.encode("utf-8"), password)
+    elif isinstance(input_data, bytes):
+        pdf = pdfium.FPDF_LoadMemDocument64(input_data, len(input_data), password)
+        ld_data = (input_data, )
+    elif is_input_buffer(input_data):
+        fileaccess, ld_data = get_fileaccess(input_data)
+        pdf = pdfium.FPDF_LoadCustomDocument(fileaccess, password)
+    else:
+        raise TypeError("Invalid input type '%s'" % type(input_data).__name__)
+    
+    if pdfium.FPDF_GetPageCount(pdf) < 1:
+        err_code = pdfium.FPDF_GetLastError()
+        pdfium_msg = ErrorToStr.get(err_code, "Error code %s" % err_code)
+        raise PdfiumError("Loading the document failed (PDFium: %s)" % pdfium_msg)
+    
+    return pdf, ld_data
+
+
 class _writer_class:
     
     def __init__(self, buffer):
@@ -541,6 +637,59 @@ class _writer_class:
         return 1
 
 
+class PdfXObject:
+    """
+    XObject helper class.
+    
+    Attributes:
+        raw (FPDF_XOBJECT): The underlying PDFium XObject handle.
+        pdf (PdfDocument): Reference to the document this XObject belongs to.
+    """
+    
+    def __init__(self, raw, pdf):
+        self.raw = raw
+        self.pdf = pdf
+        self._finalizer = weakref.finalize(
+            self, self._static_close,
+            self.raw, self.pdf,
+        )
+    
+    def _tree_closed(self):
+        if self.raw is None:
+            return True
+        return self.pdf._tree_closed()
+    
+    @staticmethod
+    def _static_close(raw, parent):
+        # logger.debug("Closing XObject")
+        if parent._tree_closed():
+            logger.critical("Document closed before XObject (this is illegal). Document: %s" % parent)
+        pdfium.FPDF_CloseXObject(raw)
+    
+    def close(self):
+        """
+        Free memory by applying the finalizer for the underlying PDFium XObject.
+        Please refer to the generic note on ``close()`` methods for details.
+        """
+        if self.raw is None:
+            logger.warning("Duplicate close call suppressed on XObject %s" % self)
+            return
+        self._finalizer()
+        self.raw = None
+    
+    def as_pageobject(self):
+        """
+        Returns:
+            PdfPageObject: A new pageobject referencing the XObject.
+        """
+        raw_pageobj = pdfium.FPDF_NewFormObjectFromXObject(self.raw)
+        return PdfPageObject(
+            raw = raw_pageobj,
+            type = pdfium.FPDF_PAGEOBJ_FORM,
+            pdf = self.pdf,
+        )
+
+
 class HarfbuzzFont:
     """ Harfbuzz font data helper class. """
     
@@ -566,12 +715,31 @@ class PdfFont:
         self.raw = raw
         self.pdf = pdf
         self._font_data = font_data
+        self._finalizer = weakref.finalize(
+            self, self._static_close,
+            self.raw, self.pdf, self._font_data,
+        )
+    
+    def _tree_closed(self):
+        if self.raw is None:
+            return True
+        return self.pdf._tree_closed()
+    
+    @staticmethod
+    def _static_close(raw, parent, font_data):
+        # logger.debug("Closing font")
+        if parent._tree_closed():
+            logger.critical("Document closed before font (this is illegal). Document: %s" % parent)
+        pdfium.FPDFFont_Close(raw)
+        id(font_data)
     
     def close(self):
         """
-        Close the font to release allocated memory.
-        This function shall be called when finished working with the object.
+        Free memory by applying the finalizer for the underlying PDFium font.
+        Please refer to the generic note on ``close()`` methods for details.
         """
-        pdfium.FPDFFont_Close(self.raw)
+        if self.raw is None:
+            logger.warning("Duplicate close call suppressed on font %s" % self)
+            return
+        self._finalizer()
         self.raw = None
-        id(self._font_data)
